@@ -1,4 +1,4 @@
 #!/bin/sh
 
 echo "Run trigger (triggered by $TRIGGER_PATH)"
-touch triggered.txt
+echo $CONTENT_FOR_FILE > triggered.txt
